crypto = require 'crypto'

genSalt = (len = 16) ->
	throw "Salt too short" if len < 8
	crypto.randomBytes(Math.ceil((len*3)/4)).toString('base64').slice 0, len

encrypt = (pass, salt, iter = 10, algo = 'sha256') ->
	hash = crypto.createHash algo
	hash.update pass + ''
	hash.update salt + ''
	
	res = ''
	
	for i in [1..Math.pow 2, iter]
		res = hash.digest 'base64'
		hash = crypto.createHash algo
		hash.update res
	
	res

makePassword = (pass, iter = 10, algo = 'sha256', saltLen = 32) ->
	throw "Only integer iterations" if typeof iter isnt 'number'
	throw "Only integer salt length" if typeof saltLen isnt 'number'
	salt = genSalt saltLen
	"#{algo}|#{iter}|#{encrypt pass, salt, iter, algo}|#{salt}"

verifyPassword = (pass, pw) ->
	dec = pw.split '|'
	throw "Invalid password format" if dec.length isnt 4
	dec[2] is encrypt pass, dec[3], dec[1], dec[0]

module.exports =
	makePassword: makePassword
	verifyPassword: verifyPassword
